﻿using ProjectAccordionBus.Drawnings;
using ProjectAccordionBus.Entities;
using ProjectAccordionBus.MovementStrategy;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectAccordionBus;
public partial class FormAccordionBus : Form
{

    private DrawningBus? _drawningBus;

    private AbstractStrategy? _strategy;


    public FormAccordionBus()
    {
        InitializeComponent();
        _strategy = null;
    }

    private void Draw()
    {
        if (_drawningBus == null)
        {
            return;
        }

        Bitmap bmp = new(pictureBoxAccordionBus.Width, pictureBoxAccordionBus.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawningBus.DrawTransport(gr);
        pictureBoxAccordionBus.Image = bmp;
    }

    private void CreateObject(string type)
    {
        Random random = new();
        switch (type)
        {
            case nameof(DrawningBus):
                _drawningBus = new DrawningBus(random.Next(100, 300), random.Next(1000, 3000),
            Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)));
                break;
            case nameof(DrawningAccordionBus):
                _drawningBus = new DrawningAccordionBus(random.Next(100, 300), random.Next(1000, 3000),
            Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
            Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
            Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }

        _drawningBus.SetPictureSize(pictureBoxAccordionBus.Width, pictureBoxAccordionBus.Height);
        _drawningBus.SetPosition(random.Next(10, 100), random.Next(10, 100));
        _strategy = null;
        comboBoxStrategy.Enabled = true;
        Draw();
    }

    private void ButtonCreate_Click(object sender, EventArgs e)
    {
        CreateObject(nameof(DrawningAccordionBus));
    }

    private void buttonCreateBus_Click(object sender, EventArgs e)
    {
        CreateObject(nameof(DrawningBus));
    }

    private void ButtonMove_Click(object sender, EventArgs e)
    {
        if (_drawningBus == null)
        {
            return;
        }

        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawningBus.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawningBus.MoveTransport(DirectionType.Down);
                break;
            case "buttonLeft":
                result = _drawningBus.MoveTransport(DirectionType.Left);
                break;
            case "buttonRight":
                result = _drawningBus.MoveTransport(DirectionType.Right);
                break;
        }

        if (result)
        {
            Draw();
        }
    }

    private void buttonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawningBus == null)
        {
            return;
        }

        if (comboBoxStrategy.Enabled)
        {
            _strategy = comboBoxStrategy.SelectedIndex switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };
            if (_strategy == null)
            {
                return;
            }
            _strategy.SetData(new MoveableBus(_drawningBus), pictureBoxAccordionBus.Width, pictureBoxAccordionBus.Height);
        }

        if (_strategy == null)
        {
            return;
        }

        comboBoxStrategy.Enabled = false;
        _strategy.MakeStep();
        Draw();

        if (_strategy.GetStatus() == StrategyStatus.Finish)
        {
            comboBoxStrategy.Enabled = true;
            _strategy = null;
        }
    }

    private void comboBoxStrategy_SelectedIndexChanged(object sender, EventArgs e)
    {

    }
}



